"""
Hacked and meant to be imported in your project.

::

    from crudlfap.settings import *
    # [...] your settings
    install_optional(OPTIONAL_APPS, INSTALLED_APPS)
    install_optional(OPTIONAL_MIDDLEWARE, MIDDLEWARE)
"""


import os

from crudlfap.conf import install_optional  # noqa


# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.abspath(__file__))))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.9/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = ')2m96!l47(!3)pvl#2@7wjh9&frxtq89*lrg(u(45gt(mnyi6d'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

# Application definition
CRUDLFAP_APPS = [
    'crudlfap',
    'betterforms',
    'bootstrap3',
    'material',
    'crudlfap_filtertables2',
    'crudlfap_auth',
    'django_filters',
    'django_tables2',
    'django.contrib.admin',
]

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
] + CRUDLFAP_APPS

# CRUDLFA+ optional dependencies
OPTIONAL_APPS = [
    # {'debug_toolbar': {'after': 'django.contrib.staticfiles'}},
    {'django_extensions': {'before': 'crudlfap'}},
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

OPTIONAL_MIDDLEWARE = [
    # {'debug_toolbar.middleware.DebugToolbarMiddleware': None}
]
INTERNAL_IPS = ('127.0.0.1',)

ROOT_URLCONF = 'crudlfap_example.urls'

TEMPLATE_CONSTANTS = {
    "settings": dict(
        SITE_NAME='CRUDLFA+DEMO',
        SITE_TITLE='CRUDLFA+ demo !',
        DEBUG=DEBUG,
    ),
}

CRUDLFAP_TEMPLATE_BACKEND = {
    "BACKEND": "django_jinja.backend.Jinja2",
    "APP_DIRS": True,
    "OPTIONS": {
        "app_dirname": "jinja2",
        "match_extension": ".html",
        "context_processors": [
            "django.template.context_processors.debug",
            "django.template.context_processors.request",
            "django.contrib.auth.context_processors.auth",
            "django.contrib.messages.context_processors.messages",
            "django.template.context_processors.i18n",
        ],
        "extensions": [
            "jinja2.ext.do",
            "jinja2.ext.loopcontrols",
            "jinja2.ext.with_",
            "jinja2.ext.i18n",
            "jinja2.ext.autoescape",
            "django_jinja.builtins.extensions.CsrfExtension",
            "django_jinja.builtins.extensions.CacheExtension",
            "django_jinja.builtins.extensions.TimezoneExtension",
            "django_jinja.builtins.extensions.UrlsExtension",
            "django_jinja.builtins.extensions.StaticFilesExtension",
            "django_jinja.builtins.extensions.DjangoFiltersExtension",
        ],
        "constants": TEMPLATE_CONSTANTS,
        "globals": {
            "pagination_filter_params": "crudlfap.jinja2.pagination_filter_params",  # noqa
            "crudlfap_site": "crudlfap.crudlfap.site",
            "getattr": getattr,
            "str": str,
            "int": int,
            "isinstance": isinstance,
            "type": type,
            "render_table": "crudlfap_filtertables2.jinja2.render_table",
            "render_form": "crudlfap.jinja2.render_form",
            "render_button": "bootstrap3.forms.render_button",
            "json": "crudlfap.jinja2.json",
        },
        "newstyle_gettext": True,
        "bytecode_cache": {
            "name": "default",
            "backend": "django_jinja.cache.BytecodeCache",
            "enabled": False,
        },
        "autoescape": True,
        "auto_reload": DEBUG,
        "translation_engine": "django.utils.translation",
    }
}

try:
    import webpack_loader  # noqa
except ImportError:
    pass
else:
    CRUDLFAP_TEMPLATE_BACKEND['OPTIONS']['globals'].setdefault(
        'render_bundle',
        'webpack_loader.templatetags.webpack_loader.render_bundle',
    )

DEFAULT_TEMPLATE_BACKEND = {
    "BACKEND": "django.template.backends.django.DjangoTemplates",
    "APP_DIRS": True,
    "OPTIONS": {
        "context_processors": [
            "django.template.context_processors.debug",
            "django.template.context_processors.request",
            "django.contrib.auth.context_processors.auth",
            "django.contrib.messages.context_processors.messages",
        ],
    },
}

TEMPLATES = [CRUDLFAP_TEMPLATE_BACKEND, DEFAULT_TEMPLATE_BACKEND]
LOGIN_REDIRECT_URL = '/'
LOCALE_PATHS = (
    os.path.join(BASE_DIR, "locale"),
)

WSGI_APPLICATION = 'crudlfap_example.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.9/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Password validation
# https://docs.djangoproject.com/en/1.9/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',  # noqa
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',  # noqa
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',  # noqa
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',  # noqa
    },
]


# Internationalization
# https://docs.djangoproject.com/en/1.9/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.9/howto/static-files/

STATIC_URL = '/static/'



from django.template.context import RequestContext
from django.template.loader import get_template
from django.template import Template

def render_table(request, table, object_list):
    """Poor man's binding to Django render_table template tag."""
    context = RequestContext(request, dict(
        table=table,
        object_list=object_list
    ))
    template = Template('''
    {% load django_tables2 %}{% render_table table %}
    ''')
    html = template.render(context)
    return html



import logging

from crudlfap import crudlfap

from django.contrib import auth
from django.contrib import messages
from django.contrib.auth.forms import PasswordChangeForm, SetPasswordForm
from django import http


logger = logging.getLogger()


class PasswordView(crudlfap.UpdateView):
    slug = 'password'
    material_icon = 'vpn_key'
    color = 'purple darken-4'
    controller = 'modal'
    action = 'click->modal#open'

    def get_form_class(self):
        if self.object == self.request.user:
            cls = PasswordChangeForm
        else:
            cls = SetPasswordForm
        # This fixes the form messages feature from UpdateView
        return type(cls.__name__, (cls,), dict(instance=self.object))

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['user'] = kwargs.pop('instance')
        return kwargs


class BecomeUser(crudlfap.ObjectView):
    urlname = 'su'
    menus = ['object']
    material_icon = 'attach_money'
    color = 'pink darken-4'

    def get_object(self, queryset=None):
        user = super().get_object()

        if user:
            user.backend = 'django.contrib.auth.backends.ModelBackend'
        else:
            messages.error(
                self.request,
                'Could not find user {}'.format(self.kwargs['username'])
            )

        return user

    def get(self, request, *a, **k):
        logger.info('BecomeUser by {}'.format(self.request.user))
        become_user = request.session.get('become_user', request.user.pk)
        new_user = self.get_object()
        if new_user:
            auth.login(request, new_user)
            request.session['become_user'] = become_user
            messages.info(
                request,
                'Switched to user {}'.format(new_user)
            )

        # FUTURE: #23 redirect to / when request.path permissions are not ok for new_user
        # return http.HttpResponse('<script type="text/javascript">history.back()</script>')
        return http.HttpResponseRedirect('/' + self.router.registry.urlpath)


class Become(crudlfap.View):
    urlname = 'su'

    def allow(self):
        return 'become_user' in self.request.session

    def get_object(self):
        user = self.model.objects.get(pk=self.request.session['become_user'])
        user.backend = 'django.contrib.auth.backends.ModelBackend'
        return user

    def get(self, request, *a, **k):
        logger.info('Become by {}'.format(self.request.user))
        if 'become_user' not in request.session:
            logger.debug('No become_user in session {}'.format(self.request.user))
            return http.HttpResponseNotFound()

        if 'become_user' in request.session:
            user = self.get_object()
            auth.login(request, user)
            messages.info(
                request,
                'Switched back to your user {}'.format(
                    user
                )
            )
            if 'become_user' in request.session.keys():
                del request.session['become_user']
        else:
            messages.warning(
                request,
                'Your are still superuser {}'.format(self.request.user)
            )

        return http.HttpResponseRedirect('/' + self.registry.urlpath)



import copy
import urllib

from django import template


def pagination_filter_params(data):
    data = copy.deepcopy(data)
    if 'page' in data:
        data.pop('page')
    return urllib.parse.urlencode(data)


def link(url_name, html, attributes=None):
    pass


def get_view(url_name):
    pass


def json(arg):
    import json
    return json.dumps(arg)


def render_form(form):
    return template.Template(
        '{% load material_form %}{% form form=form %}{% endform %}'
    ).render(template.Context(dict(form=form)))



import datetime

from django.conf import settings
from django.contrib import messages
from django.core.cache import cache
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

import timeago


class LockViewMixin(object):
    """Currently implements soft lock."""
    locks = False

    def get_deadlock_delta(self):
        return datetime.timedelta(**self.deadlock_delta_kwargs)

    def get_deadlock_delta_kwargs(self):
        return dict(seconds=30)

    def get_lock_key(self):
        return 'lock-{}'.format(self.url)

    def get_lock_value(self):
        return cache.get(self.lock_key)

    def get_locked(self):
        if not self.locks:
            return False
        value = self.lock_value
        if not value:
            return False
        if value['datetime'] + self.deadlock_delta < timezone.now():
            # expired lock
            cache.delete(self.lock_key)
            return False
        return True

    def get_locked_message(self):
        return _('Page was open by {} {}').format(
            self.lock_value.get('username', _('unknown user')),
            timeago.format(
                timezone.now() - self.lock_value['datetime'],
                getattr(self.request, 'LANGUAGE_CODE', settings.LANGUAGE_CODE)
            )
        )

    def get(self, request, *args, **kwargs):
        if self.locked:
            messages.warning(request, self.locked_message)
        if self.locks:
            self.lock()
        return super().get(request, *args, **kwargs)

    def lock(self):
        value = dict(datetime=timezone.now())
        if self.request.user.is_authenticated:
            value['username'] = self.request.user.username
        cache.set(self.lock_key, value)



from crudlfap import crudlfap

from .models import Song


class SongRouter(crudlfap.Router):
    fields = '__all__'
    icon = 'music'
    model = Song

    views = [
        crudlfap.DeleteView,
        crudlfap.UpdateView,
        crudlfap.CreateView,
        crudlfap.DetailView,
        crudlfap.FilterTables2ListView.clone(
            filter_fields=['artist'],
            search_fields=['artist__name', 'name'],
        ),
    ]

    def allowed(self, view):
        # In this example, we let users do everything on their own objects
        # and that's taken care of by get_objects_for_user. So, we short-
        # circuit the Django permission system which takes place by default
        # We only have the create view that does not go through
        # get_objects_for_user, deal with that
        return view.request.user.is_authenticated

    def get_objects_for_user(self, user, perms):
        if not user.is_authenticated:
            return self.model.objects.none()

        if user.is_staff or user.is_superuser:
            return self.model.objects.all()

        return self.model.objects.filter(owner=user)

SongRouter().register()



# Generated by Django 2.0.3 on 2018-04-15 15:22

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('artist', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Song',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='title')),
                ('duration', models.IntegerField(default=320)),
                ('artist', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='artist.Artist')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('name',),
            },
        ),
    ]



"""
CRUDLFA+ router for Django 2.0.

Note that you can also use non-database backed models, by inheriting from
models.Model and setting their Meta.managed attribute to False. Then, you can
use CRUDLFA+ views and routers.
"""
from django.apps import apps
from django.urls import path
from django.utils.module_loading import import_string

from .route import Route
from .utils import guess_urlfield


crudlfap = apps.get_app_config('crudlfap')  # pylint: disable=invalid-name


class ViewsDescriptor(object):
    def __init__(self, default=None):
        self.default = default or []

    def __get__(self, obj, objtype):
        if getattr(obj, '_views', None) is None:
            if callable(self.default):
                obj.views = self.default()
            else:
                obj.views = self.default
        return obj._views

    def __set__(self, obj, value):
        obj._views = Views(value)


class Views(list):
    def __getitem__(self, key):
        if isinstance(key, int):
            return super().__getitem__(key)

        for view in self:
            if view.urlname == key:
                return view

    def __setitem__(self, key, value):
        if isinstance(key, int):
            return super().__setitem__(key, value)

        for i, view in enumerate(self):
            if view.urlname == key:
                return super().__setitem__(i, value)

    def __delitem__(self, key):
        if isinstance(key, int):
            return super().__delitem__(key)

        for i, view in enumerate(self):
            if view.urlname == key:
                return super().__delitem__(i)


class Router(object):
    """
    Base router for CRUDLFA+ Route.

    .. py:attribute:: model

        Optional model class for this Router and all its views.
    """
    views = ViewsDescriptor(crudlfap.get_default_model_views)

    def __getattr__(self, attr):
        if attr.startswith('get_'):
            raise AttributeError('{} or {}()'.format(attr[4:], attr))

        if hasattr(self, 'get_' + attr):
            return getattr(self, 'get_' + attr)()

        raise AttributeError('{} or get_{}()'.format(attr, attr))

    def get_urlfield(self):
        return guess_urlfield(self.model)

    def get_namespace(self):
        if self.model:
            return self.model._meta.model_name

    def get_urlpath(self):
        if self.model:
            return self.model._meta.model_name

    def get_app_name(self):
        return self.model._meta.app_label

    def get_registry(self):
        from crudlfap.crudlfap import site
        return site

    def __getitem__(self, urlname):
        """Get a view by urlname."""
        for view in self.views:
            if view.urlname == urlname:
                return view

        raise KeyError(
            'View with urlname {} not in router {}'.format(
                urlname,
                type(self).__name__,
            )
        )

    def generate_views(self, *views):
        """
        Generate views for this router, core of the automation in CRUDLFA+.

        This method considers each view in given args or self.views and returns
        a list of usable views.

        Each arg may be a view class or a dict of attributes with a `_cls` key
        for the actual view class.

        It will copy the view class and bind the router on it in the list this
        returns.

        For example, this would cause two view classes to be returned, if
        self.model is ``Artist``, then ``CreateView`` will be used as parent to
        create ``ArtistCreateView`` and ``DetailView`` will be used to create
        ``ArtistDetailView``, also setting the attribute
        ``extra_stuff='bar'``::

            Router(Artist).generate_views([
                CreateView,
                dict(_cls=DetailView, extra_stuff='bar'),
                ListView.factory(paginate_by=12),
            ])
        """
        result = []
        for arg in views or self.views:
            view = arg

            if isinstance(view, str):
                view = import_string(view)

            try:
                if not issubclass(view, Route) and view != Route:
                    view = type(view.__name__, (view, Route), {})
            except Exception as e:
                print('Got an error with view:', view)
                raise e

            view = view.clone(model=self.model, router=self)

            result.append(view)
        return result

    def __init__(self, model=None, registry=None, views=None, **attributes):
        """Create a Router for a Model."""

        if model is not None:
            self.model = model

        if registry is not None:
            self.registry = registry

        '''
        if fields is None:
            warnings.warn(
                '{} has no fields, defaulting to __all__ !'.format(self)
            )
            fields = '__all__'

        self.fields = fields
        '''

        for name, value in attributes.items():
            setattr(self, name, value)

        self.views = Views(self.generate_views(*(views or [])))

    def register(self):
        """
        Register to self.registry.

        Also, adds the get_absolute_url() method to the model class if it has
        None, to return the reversed url for this instance to the view of this
        Router with the ``detail`` slug.

        Set get_absolute_url in your model class to disable this feature. Until
        then, welcome in 2018.

        Also, register this router as default router for its model class in the
        RouterRegistry.
        """
        self.registry[self.model] = self

        for view in self.views:
            if view.urlname == 'detail':
                break
        if not self.views or view.urlname != 'detail':
            return

        if not hasattr(self.model, 'get_absolute_url'):
            def get_absolute_url(self):
                from crudlfap import crudlfap
                return crudlfap.site[type(self)]['detail'].clone(
                    object=self).url
            self.model.get_absolute_url = get_absolute_url

    def get_urlpatterns(self):
        """
        Generate URL patterns for this Router views.
        """
        return [view.urlpattern for view in self.views]

    def get_urlpattern(self):
        return path(self.urlpath, (
            [
                path('/', (
                    [v.urlpattern for v in self.views if v.urlpath],
                    None,
                    None
                ))
            ] + [v.urlpattern for v in self.views if not v.urlpath],
            self.app_name,
            self.namespace,
        ))

    def get_menu(self, name, request, **kwargs):
        """
        Return allowed view objects which have ``name`` in their ``menus``.

        For each view class in self.views which have ``name`` in their
        ``menus`` attribute, instanciate the view class with ``request`` and
        kwargs, call ``allowed()`` on it.

        Return the list of view instances for which ``allowed()`` has passed.
        """
        views = []

        for v in self.views:
            if name not in getattr(v, 'menus', []):
                continue

            view = v.clone(request=request, **kwargs)()

            if view.allowed:
                views.append(view)

        return views

    def get_model(self):
        return None

    def allowed(self, view):
        """
        Return True to allowed a access to a view.

        Called by the default view.allowed() implementation.

        If you override the view.allowed() method, then it's up to you
        to decide if you want to call this method or not.

        Returns True if user.is_staff by default.
        """
        if view.required_permissions:
            for permission in view.required_permissions:
                args = [view.object] if view.object_permission_check else []
                if not view.request.user.has_perm(permission, *args):
                    return False
        else:
            return view.request.user.is_staff
        return True

    def get_objects_for_user(self, user, perms):
        """Return the list of objects for a given set of perms."""
        return self.model.objects.all()

    def get_fields_for_user(self, user, perms, obj=None):
        """Return the list of fields for a user."""
        return [f.name for f in self.model._meta.fields]



"""Import everything we expose in crudlfap namespace."""

from django.utils.module_loading import import_string

from .apps import _installed
from .factory import Factory
from .registry import Registry
from .route import Route
from .router import Router, Views, ViewsDescriptor
from .site import site
from .views.debug import UrlsView
from .views.generic import (
    CreateView,
    DeleteView,
    DetailView,
    FormView,
    HistoryView,
    ListView,
    ModelViewMixin,
    ObjectFormView,
    ObjectView,
    ObjectViewMixin,
    TemplateView,
    UpdateView,
    View,
    ViewMixin,
)
from .views.lock import LockViewMixin

if _installed('crudlfap_filtertables2'):
    from crudlfap_filtertables2.views import FilterTables2ListView



from crudlfap.views import generic
from crudlfap_filtertables2.views import FilterTables2ListView


class HistoryView(generic.ObjectView, FilterTables2ListView):
    menus = ['object']
    material_icon = 'history'

    def get_filterset_kwargs(self):
        return {
            'data': self.request.GET or None,
            'request': self.request,
            'queryset': self.get_queryset(),
        }



from django.db import models


class SongRatingManager(models.Manager):
    object_list = []


class SongRating(models.Model):
    song = models.ForeignKey('song.Song', on_delete=models.DO_NOTHING)
    rating = models.IntegerField()

    objects = SongRatingManager()

    class Meta:
        managed = False

    def __str__(self):
        return '{}: {}'.format(self.song, self.rating)

    def save(self):
        if self.pk:
            return  # already in memory

        object_list = type(self).objects.object_list
        self.pk = len(object_list) + 1
        object_list.append(self)

    def delete(self):
        type(self).objects.object_list = [
            o for o in type(self).objects.object_list
            if o.pk != self.pk
        ]



class Factory(object):
    @classmethod
    def clone(cls, **attributes):
        """Return a subclass with the given attributes.

        If a model is found, it will prefix the class name with the model.
        """
        name = cls.__name__
        model = attributes.get('model', getattr(cls, 'model', None))
        if model is None:
            model = getattr(cls, 'model', None)
        if model and model.__name__ not in cls.__name__:
            name = '{}{}'.format(model.__name__, cls.__name__)
        return type(name, (cls,), attributes)



"""
Test security with crudlfap_example.song setup.

It's a bit paranoid, but i'll sleep better with this: i don't trust users, i
don't trust myself either :)
"""
from datetime import timedelta

from crudlfap import crudlfap

from crudlfap_auth.crudlfap import User

from crudlfap_example.blog.models import Post

from django.utils import timezone

import pytest


def user(**attrs):
    user, created = User.objects.get_or_create(**attrs)
    if created:
        user.set_password('password')
        user.save()
    return user


def user_client(client, **userattrs):
    if not userattrs.get('username', None):
        return client
    user(**userattrs)
    if not client.login(username=userattrs['username'], password='password'):
        raise Exception('Could not auth as ' + userattrs['username'])
    return client


router = pytest.fixture(lambda: crudlfap.site[Post])
now = timezone.now()
yesterday = now - timedelta(days=1)
tomorrow = now + timedelta(days=1)

user0 = pytest.fixture(lambda: user(username='user0'))
user1 = pytest.fixture(lambda: user(username='user1'))
staff = pytest.fixture(lambda: user(username='staff', is_staff=True))

post0 = pytest.fixture(lambda user0: Post.objects.get_or_create(
    owner=user0, name='post0', publish=yesterday)[0])
post1 = pytest.fixture(lambda user0: Post.objects.get_or_create(
    owner=user0, name='post1', publish=tomorrow)[0])
post2 = pytest.fixture(lambda user1: Post.objects.get_or_create(
    owner=user1, name='post2', publish=yesterday)[0])

url = pytest.fixture(lambda name, obj: router()[name].clone(object=obj).url)


@pytest.mark.django_db
def test_list_view(router, client, user0, user1, post0, post1, post2):
    list_url = router['list'].url
    res = user_client(client, username='user0').get(list_url)
    assert b'post0' in res.content
    assert b'post1' in res.content
    assert b'post2' in res.content
    for name in ('update', 'delete'):
        view = router[name]
        assert str(view.clone(object=post0).url) in str(res.content)
        assert str(view.clone(object=post1).url) in str(res.content)
        assert str(view.clone(object=post2).url) not in str(res.content)

    res = user_client(client, username='user1').get(list_url)
    assert b'post0' in res.content
    assert b'post1' not in res.content
    assert b'post2' in res.content
    for name in ('update', 'delete'):
        view = router[name]
        assert str(view.clone(object=post0).url) not in str(res.content)
        assert str(view.clone(object=post1).url) not in str(res.content)
        assert str(view.clone(object=post2).url) in str(res.content)


@pytest.mark.django_db
def test_anonymous_detail_published(router, client, post0, post1):
    assert client.get(url('detail', post0)).status_code == 200
    assert client.get(url('detail', post1)).status_code == 404


@pytest.mark.django_db
def test_owner_detail_unpublished(router, client, post0, post1):
    client = user_client(client, username='user0')
    assert client.get(url('detail', post0)).status_code == 200
    assert client.get(url('detail', post1)).status_code == 200


@pytest.mark.django_db
def test_nonowner_detail_unpublished(router, client, post0, post1):
    client = user_client(client, username='user1')
    assert client.get(url('detail', post0)).status_code == 200
    assert client.get(url('detail', post1)).status_code == 404


@pytest.mark.django_db
def test_anonymous_edit_published(router, client, post0, post1):
    for name in ('delete', 'update'):
        assert client.get(url(name, post0)).status_code == 404
        assert client.get(url(name, post1)).status_code == 404


@pytest.mark.django_db
def test_owner_edits(router, client, post0, post1, post2):
    client = user_client(client, username='user0')
    for name in ('delete', 'update'):
        assert client.get(url(name, post0)).status_code == 200
        assert client.get(url(name, post1)).status_code == 200
        assert client.get(url(name, post2)).status_code == 404



"""Django AppConfig for the crudlfap module."""

from django import apps
from django.conf import settings


def _installed(*apps):
    for app in apps:
        if app not in settings.INSTALLED_APPS:
            return False
    return True


class DefaultConfig(apps.AppConfig):
    """Default AppConfig."""

    name = 'crudlfap'

    def get_default_model_views(self):  # pylint: disable=no-self-use
        """Return the default views to use in a new router."""
        crudlfap = getattr(settings, 'CRUDFLAP', {})

        if _installed('crudlfap_filtertables2'):
            list_view = 'crudlfap_filtertables2.views.FilterTables2ListView'
        else:
            list_view = 'crudlfap.views.generic.ListView'

        views = crudlfap.get(
            'default_views',
            [
                'crudlfap.views.generic.CreateView',
                'crudlfap.views.generic.DeleteView',
                'crudlfap.views.generic.UpdateView',
                'crudlfap.views.generic.DetailView',
                list_view,
            ]
        )

        return views

    def ready(self):
        super().ready()
        self.module.autodiscover()



import collections

from betterforms.changelist import SearchForm
from crudlfap.views.generic import ListView

from django import template
from django.db import models
from django.utils.translation import ugettext_lazy as _

import django_filters
from django_filters.filterset import filterset_factory

import django_tables2 as tables
from django_tables2.config import RequestConfig


class Table(tables.Table):
    def render_crudlfap(self, record):
        from django.template import loader
        from crudlfap import crudlfap
        context = dict(
            object=record,
            views=crudlfap.site[type(record)].get_menu(
                'object',
                self.context['request'],
                object=record,
            ),
            extra_class='btn-small secondary-content'
        )
        template = loader.select_template([
            '{}/_{}_actions.html'.format(
                type(record)._meta.app_label,
                type(record)._meta.model_name,
                ),
            'crudlfap/_actions.html',
            ])
        return template.render(context)

    def render_tags(self, record):
        html = []
        for tag in record.tags.all():
            html.append(unicode(tag))

        return ' '.join(html)


class FilterMixin(object):
    def get_filterset_class(self):
        return filterset_factory(model=self.model, fields=self.filter_fields)

    def get_filterset(self):
        """
        Returns an instance of the filterset to be used in this view.
        """
        fs = self.filterset_class(**self.filterset_kwargs)

        # filter out choices which have no result to avoid filter pollution
        # with choices which would empty out results
        for name, field in fs.form.fields.items():
            try:
                mf = self.model._meta.get_field(name)
            except:
                continue

            if not isinstance(mf, models.ForeignKey):
                continue

            field.queryset = field.queryset.annotate(
                c=models.Count(mf.related_query_name())
            ).filter(c__gt=0)

        return fs

    def get_filterset_kwargs(self):
        """
        Returns the keyword arguments for instanciating the filterset.
        """
        return {
            'data': self.request.GET or None,
            'request': self.request,
            'queryset': self.get_queryset(),
        }

    def get_filterset_meta_filter_overrides(self):
        return {
            models.CharField: {
               'filterset_class': django_filters.CharFilter,
               'extra': lambda f: {
                   'lookup_expr': 'icontains',
               },
           },
        }

    def get_filter_fields(self):
        return []

    def get_filterset_meta_attributes(self):
        return dict(
            model=self.model,
            fields=self.filter_fields,
            filter_overrides=self.filterset_meta_filter_overrides
        )

    def get_filterset_meta_class(self):
        return type('Meta', (object,), self.filterset_meta_attributes)

    def get_filterset_class_attributes(self):
        return dict(Meta=self.filterset_meta_class)

    def get_filterset_class(self):
        return type(
            '{}FilterSet'.format(self.model.__name__),
            (django_filters.FilterSet,),
            self.filterset_class_attributes
        )


class TableMixin(object):
    def get_table_fields(self):
        if self.table_sequence:
            return [
                f.name
                for f in self.model._meta.fields
                if f.name in self.table_sequence
            ]

        return [
            f.name
            for f in self.model._meta.fields
            if f.name not in self.exclude
        ]

    def get_table_link_fields(self):
        if not hasattr(self.model, 'get_absolute_url'):
            return []

        for field in self.table_fields:
            model_field = self.model._meta.get_field(field)
            if isinstance(model_field, models.CharField):
                return [field]

        return []

    def get_table_meta_link_columns(self):
        return {i: tables.LinkColumn() for i in self.table_link_fields}

    def get_table_meta_action_columns(self):
        return dict(
            crudlfap=tables.TemplateColumn(
                template_name='crudlfap/_actions.html',
                verbose_name=_('Actions'),
                extra_context=dict(extra_class='btn-small'),
                orderable=False,
            ),
        )

    def get_table_sequence(self):
        return None

    def get_table_columns(self):
        return dict()

    def get_table_meta_attributes(self):
        attrs = dict(model=self.model)

        if self.table_sequence:
            attrs['sequence'] = self.table_sequence

        if self.table_fields:
            attrs['fields'] = self.table_fields

        return attrs

    def get_table_meta_class(self):
        return type('Meta', (object,), self.table_meta_attributes)

    def get_table_class_attributes(self):
        attrs = collections.OrderedDict(
            Meta=self.table_meta_class,
        )
        attrs.update(self.table_meta_link_columns)
        attrs.update(self.table_meta_action_columns)
        attrs.update(self.table_columns)
        return attrs

    def get_table_class(self):
        return Table

    def build_table_class(self):
        bases = (self.table_class,)
        if (self.table_class != Table
                and not issubclass(self.table_class, Table)):

            bases = (self.table_class, Table)

        return type(
            '{}Table'.format(self.model.__name__),
            bases,
            self.table_class_attributes
        )

    def get_table_kwargs(self):
        return {}

    def get_table_pagination(self):
        if not self.paginate_by:
            return True
        return dict(per_page=self.paginate_by)

    def get_table(self):
        kwargs = self.table_kwargs
        kwargs.update(data=self.object_list)
        self.table = self.build_table_class()(**kwargs)
        RequestConfig(
            self.request,
            paginate=self.table_pagination,
        ).configure(self.table)
        return self.table


class SearchMixin(object):
    def get_search_fields(self):
        if hasattr(self.router, 'search_fields'):
            return self.router.search_fields
        return [
            f.name
            for f in self.model._meta.fields
            if isinstance(f, models.CharField)
        ]

    def get_search_form_class(self):
        if not self.search_fields:
            return

        return type(
            self.model.__name__ + 'SearchForm',
            (SearchForm,),
            dict(
                SEARCH_FIELDS=self.search_fields,
                model=self.model
            )
        )


class FilterTables2ListView(SearchMixin, FilterMixin, TableMixin, ListView):
    urlre = r'$'
    default_template_name = 'crudlfap_filtertables2/list.html'
    icon = 'fa fa-fw fa-table'
    urlname = 'list'

    def get(self, request, *args, **kwargs):
        if self.filterset:
            self.object_list = self.filterset.qs
        else:
            self.object_list = self.get_queryset()

        if self.search_fields:
            self.search_form = self.get_search_form()
            self.object_list = self.search_form.get_queryset()

        # Trick super()
        self.get_queryset = lambda *a: self.object_list
        return super().get(request, *args, **kwargs)

    def get_search_form(self):
        if self.search_fields:
            form = self.search_form_class(
                self.request.GET,
                queryset=self.object_list
            )
            form.full_clean()
            return form



"""
Crudlfa+ generic views and mixins.

Crudlfa+ takes views further than Django and are expected to:

- generate their URL definitions and reversions,
- check if a user has permission for an object,
- declare the names of the navigation menus they belong to.
"""
from crudlfap.route import Route

from django.contrib import messages
from django.contrib.admin.models import ADDITION, CHANGE, DELETION, LogEntry
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from django.views import generic
from django.views.generic.detail import SingleObjectMixin

from .lock import LockViewMixin


class DefaultTemplateMixin(object):
    """
    Override for get_template_names to append default_template_name.

    This allows to configure "last resort" templates for each class, and thus
    to provide a working CRUD out of the box.
    """

    style = 'default'
    fa_icon = 'question'
    material_icon = 'priority high'
    ajax = '#ajax-container'

    def get_title(self):
        return _(self.urlname).capitalize()

    def get_title_html(self):
        """Return text for HTML title tag."""
        return self.title

    def get_title_heading(self):
        """Return text for page heading."""
        return self.title

    def get_template_names(self):
        """Give a chance to default_template_name."""
        template_names = super().get_template_names()
        default_template_name = getattr(self, 'default_template_name', None)
        if default_template_name:
            template_names.append(default_template_name)
        return template_names


class ViewMixin(LockViewMixin, DefaultTemplateMixin, Route):
    """Base View mixin for CRUDLFA+.

    If you have any question about style then find your answers in
    DefaultTemplateMixin, otherwise in RoutableViewMixin.
    """
    def get_menu(self):
        return None

    def get_menu_kwargs(self):
        return dict()

    def get_menu_views(self):
        views = []
        for view in self.router.views:
            for menu in view.menus:
                if menu in self.menus_display:
                    view = view.clone(
                        request=self.request,
                        **self.menu_kwargs,
                    )

                    if not view().allowed:
                        continue
                    if view.urlname == self.urlname:
                        continue
                    if view.urlname in [v.urlname for v in views]:
                        continue
                    views.append(view)
        return views


class View(ViewMixin, generic.View):
    """Base view for CRUDLFA+."""


class TemplateView(ViewMixin, generic.TemplateView):
    """TemplateView for CRUDLFA+."""


class ModelViewMixin(ViewMixin):
    """Mixin for views using a Model class but no instance."""

    menus = ['model']
    menus_display = ['model']
    pluralize = False
    object_permission_check = False

    def get_exclude(self):
        return []

    def get_required_permissions(self):
        return [
            '{}.{}_{}'.format(
                self.urlname,
                self.app_name,
                self.model._meta.model_name
            )
        ]

    def get_fields(self):
        return [
            f for f in self.router.get_fields_for_user(
                self.request.user,
                self.required_permissions
            )
            if self.model._meta.get_field(f).editable
            and f not in self.exclude
        ]

    def get_model_verbose_name(self):
        if self.pluralize:
            return self.model._meta.verbose_name_plural
        else:
            return self.model._meta.verbose_name

    def get_title(self):
        return '{} {}'.format(
            _(self.urlname),
            self.model_verbose_name,
        ).capitalize()

    def get_queryset(self):
        """Return router.get_queryset() by default, otherwise super()."""
        if self.router:
            return self.router.get_objects_for_user(
                self.request.user,
                self.required_permissions,
            )
        return super().get_queryset()


class ObjectMixin(object):
    """
    Make self.object call and cache self.get_object() automatically.

    WHAT A RELIEF

    However, if it has a router with the get_object() method, use it.
    """

    def get_object(self):
        """Return router.get_object() by default, otherwise super()."""
        router = getattr(self, 'router', None)
        if router and getattr(router, 'get_object', None):
            return router.get_object(self)

        if getattr(self, 'kwargs', False) is False:
            # This happens when the view has not been instanciated with an
            # object, neither from a URL which would allow getting the object
            # in the super() call below.
            raise Exception('Must instanciate the view with an object')
        return super().get_object()

    def object_get(self):
        """Return the object, uses get_object() if necessary."""
        cached = getattr(self, '_object', None)
        if not cached:
            self._object = self.get_object()
        return self._object

    def object_set(self, value):
        """Set self.object attribute."""
        self._object = value

    object = property(object_get, object_set)


class ObjectViewMixin(ObjectMixin, ModelViewMixin, SingleObjectMixin):
    """Mixin for views using a Model instance."""

    menus = ['object', 'object_detail']
    menus_display = ['object', 'object_detail']
    object_permission_check = True

    def get_urlargs(self):
        """Return list with object's urlfield attribute."""
        return [getattr(self.object, self.urlfield)]

    def get_slug_field(self):
        """Replace Django's get_slug_field with get_url_field."""
        return self.urlfield

    @property
    def slug_url_kwarg(self):
        """Replace Django's slug_url_kwarg with get_url_field."""
        return self.urlfield

    @classmethod
    def to_url_args(cls, *args):
        """Return first arg's url_field attribute."""
        url_field = cls.get_url_field()
        return [getattr(args[0], url_field)]

    @classmethod
    def get_urlpath(cls):
        """Identify the object by slug or pk in the pattern."""
        return r'<{}>/{}'.format(cls.urlfield, cls.urlname)

    def get_title(self):
        return '{} {} "{}"'.format(
            _(self.urlname),
            self.model_verbose_name,
            self.object
        ).capitalize()

    def get_menu_kwargs(self):
        return dict(object=self.object)


class ObjectView(ObjectViewMixin, View):
    pass


class FormViewMixin(ViewMixin):
    """Mixin for views which have a Form."""
    success_url_next = True

    def get_success_url(self):
        if self.success_url_next and '_next' in self.request.POST:
            return self.request.POST['_next']
        if self.object and hasattr(self.object, 'get_absolute_url'):
            return self.object.get_absolute_url()
        if self.router['list']:
            return self.router['list'].url
        return super().get_success_url()


class FormView(FormViewMixin, generic.FormView):
    """Base FormView class."""

    style = 'warning'
    default_template_name = 'crudlfap/form.html'


class ModelFormViewMixin(ModelViewMixin, FormViewMixin):
    """ModelForm ViewMixin using readable"""
    log_action_flag = False
    menus = ['model']

    def get_form_fields(self):
        if hasattr(self.router, 'form_fields'):
            return self.router.form_fields
        if hasattr(self.router, 'fields'):
            return self.router.fields
        return self.get_fields()

    def get_form_class(self):
        if self.fields is None and not self.form_class:
            self.fields = self.form_fields
        return super().get_form_class()

    def get_form_invalid_message(self):
        return '{} {} error'.format(
            self.urlname,
            self.model_verbose_name
        ).capitalize()

    def get_form_valid_message(self):
        return _(
            '%s %s: {}' % (self.urlname, self.model_verbose_name)
        ).format(self.form.instance).capitalize()

    def form_invalid(self, form):
        response = super().form_invalid(form)
        messages.error(self.request, self.form_invalid_message)
        return response

    def get_log_message(self):
        return _(self.urlname)

    def log_insert(self):
        if not self.request.user.is_authenticated:
            return

        if not self.log_action_flag:
            return

        LogEntry.objects.log_action(
            self.request.user.pk,
            ContentType.objects.get_for_model(self.model).pk,
            self.object.pk,
            str(self.object),
            self.log_action_flag,
            self.log_message,
        )

    def form_valid(self, form):
        response = super().form_valid(form)
        messages.success(self.request, self.form_valid_message)
        self.log_insert()
        return response


class ObjectFormViewMixin(ObjectViewMixin, ModelFormViewMixin):
    """Custom form view mixin on an object."""
    log_action_flag = CHANGE


class ObjectFormView(ObjectFormViewMixin, generic.FormView):
    """Custom form view on an object."""


class CreateView(ModelFormViewMixin, generic.CreateView):
    """View to create a model object."""

    style = 'success'
    material_icon = 'add'
    default_template_name = 'crudlfap/create.html'
    controller = 'modal'
    action = 'click->modal#open'
    color = 'green'
    object_permission_check = False
    log_action_flag = ADDITION

    def get_form_fields(self):
        if hasattr(self, 'create_fields'):
            return self.create_fields
        if hasattr(self.router, 'create_fields'):
            return self.router.create_fields
        return super().get_form_fields()


class DeleteView(ObjectFormViewMixin, generic.DeleteView):
    """View to delete a model object."""

    default_template_name = 'crudlfap/delete.html'
    style = 'danger'
    fa_icon = 'trash'
    material_icon = 'delete'
    success_url_next = True
    controller = 'modal'
    action = 'click->modal#open'
    color = 'red'
    log_action_flag = DELETION

    def get_success_message(self):
        return _(
            '%s %s: {}' % (self.urlname, self.model_verbose_name)
        ).format(self.object).capitalize()

    def get_success_url(self):
        messages.success(self.request, self.success_message)
        return self.router['list'].reverse()

    def get_required_permissions(self):
        return ['{}.delete_{}'.format(
            self.app_name, self.model._meta.model_name)]


class DetailView(ObjectViewMixin, generic.DetailView):
    """Templated model object detail view which takes a field option."""

    fa_icon = 'search-plus'
    material_icon = 'search'
    default_template_name = 'crudlfap/detail.html'
    color = 'blue'
    menus_display = ['object', 'object_detail']

    def get_title(self):
        return str(self.object)

    def get_context_data(self, *a, **k):
        c = super(DetailView, self).get_context_data(*a, **k)
        c['fields'] = [
            {
                'field': self.model._meta.get_field(field),
                'value': getattr(self.object, field)
            }
            for field in (
                [f.name for f in self.model._meta.fields]
                if self.fields == '__all__'
                else self.fields
            ) if field not in self.exclude
        ]
        return c

    @classmethod
    def get_urlpath(cls):
        """Identify the object by slug or pk in the pattern."""
        return r'<{}>'.format(cls.urlfield)

    def get_required_permissions(self):
        return ['{}.detail_{}'.format(
            self.app_name, self.model._meta.model_name)]


class ListView(ModelViewMixin, generic.ListView):
    """Model list view."""

    default_template_name = 'crudlfap/list.html'
    urlpath = ''
    fa_icon = 'table'
    material_icon = 'list'
    menus = ['main', 'model']
    pluralize = True

    def get(self, *a, **k):
        '''Enforce sane default paginate_by if not False.'''
        if getattr(self, 'paginate_by', None) is None:
            self.paginate_by = self.get_paginate_by()
        return super().get(*a, **k)

    def get_paginate_by(self, queryset=None):
        if self.router and hasattr(self.router, 'paginate_by'):
            return self.router.paginate_by

        return 10


class UpdateView(ObjectFormViewMixin, generic.UpdateView):
    """Model update view."""

    material_icon = 'edit'
    default_template_name = 'crudlfap/update.html'
    controller = 'modal'
    action = 'click->modal#open'
    color = 'orange'
    locks = True

    def get_form_fields(self):
        if hasattr(self, 'update_fields'):
            return self.update_fields
        if hasattr(self.router, 'update_fields'):
            return self.router.update_fields
        return super().get_form_fields()

    def get_required_permissions(self):
        return ['{}.change_{}'.format(
            self.app_name, self.model._meta.model_name)]


class HistoryView(ObjectViewMixin, generic.DetailView):
    material_icon = 'history'
    template_name_suffix = '_history'
    default_template_name = 'crudlfap/history.html'
    controller = None
    action = None

    def get_object_list(self):
        ctype = ContentType.objects.get_for_model(self.model)
        return LogEntry.objects.filter(
            content_type=ctype,
            object_id=self.object.pk,
        )



import inspect

from django import http
from django.urls import path, reverse, reverse_lazy
from django.utils.module_loading import import_string

from .factory import Factory
from .utils import guess_urlfield


class RouteMetaclass(type):
    router = None

    def __getattr__(self, attr):
        if attr.startswith('get_'):
            raise AttributeError('{} or {}'.format(attr[4:], attr))

        getter = getattr(self, 'get_' + attr)

        if inspect.ismethod(getter):
            return getter()
        else:
            return getter(self)

    def get_app_name(self):
        return self.model._meta.app_label if self.model else None

    def get_model(self):
        return self.router.model if self.router else None

    def get_urlpath(self):
        return self.urlname

    def get_urlname(self):
        urlname = self.__name__.lower()
        if urlname.endswith('view'):
            urlname = urlname[:-4]
        elif urlname.endswith('route'):
            urlname = urlname[:-5]

        if self.model:
            model_name = self.model._meta.model_name.lower()
            if urlname.startswith(model_name):
                urlname = urlname[len(model_name):]

        if not urlname and self.model:
            urlname = self.model._meta.model_name

        return urlname or None

    def get_urlpattern(self):
        return path(self.urlpath, self.as_view(), name=self.urlname)

    def get_urlfullname(self):
        if self.router and self.registry:
            return '{}:{}:{}'.format(
                self.router.registry.app_name,
                self.router.namespace,
                self.urlname
            )
        elif self.registry:
            return '{}:{}'.format(
                self.registry.app_name,
                self.urlname
            )
        elif self.router:
            return '{}:{}'.format(
                self.router.namespace,
                self.urlname
            )
        else:
            return self.urlname

    def get_urlfield(self):
        if self.router and self.router.urlfield:
            return self.router.urlfield
        return guess_urlfield(self.model)


class Route(Factory, metaclass=RouteMetaclass):
    @classmethod
    def reverse(cls, *args, **kwargs):
        """Reverse a url to this view with the given args."""
        return reverse_lazy(cls.urlfullname, args=args, kwargs=kwargs)

    def get_urlargs(self):
        """
        Return args for reversing this view url from self.
        See ``self.reverse()`` for detail.
        """
        return []

    def get_url(self):
        """
        Return the URL for this view given its current state.
        Given that the ``reverse()`` method is a class method, this should
        allow things like::
            url = YourView(object=your_object).url
        """
        return self.reverse(*self.urlargs)

    def __getattr__(self, attr):
        if attr.startswith('get_'):
            raise AttributeError('{} or {}()'.format(attr[4:], attr))

        if hasattr(self, 'get_' + attr):
            return getattr(self, 'get_' + attr)()

        return getattr(type(self), attr)

    def get_required_permissions(self):
        return None

    def get_allowed(self):
        """
        Must return True if it dispatching this view is allowed.

        This means that the view should be instanciated with all the context it
        needs for this method to work.

        The best way for you to understand the pattern being this is this
        pseudo code::

            class SomeDetailView(DetailView):
                def get_context_data(self):
                    return dict(
                        # For the object used in this view instance, we would
                        # also have been allowed to execute the delete view
                        # with this request.
                        can_delete=SomeDeleteView(
                            object=self.object,
                            request=self.request,
                        ).allowed()
                    )

        This means that your view classes are ready for an allow() override
        which considers that the view object was instanciated with everything
        it would need to dispatch.

        By default, this proxies the router's allowed() method which
        returns True for staff users by default.  If the view has no router
        then it returns True if the request user is_staff.

        Override with a lambda in a factory if you want to open to all::

            YourView.factory(allowed=True)

        You can also set allow on the router if you want to allow a whole
        router::

            YourRouter(YourModel, allowed=True)
        """
        if not self.router:
            return self.request.user.is_staff

        if not self.router.allowed(self):
            return False

        return True

    def get_registry(self):
        if self.router:
            return self.router.registry
        from .site import site
        return site

    def get_login_url(self):
        if self.registry:
            return reverse('{}:login'.format(self.registry.app_name))
        return reverse('login')

    def dispatch(self, request, *args, **kwargs):
        """Run allow() before dispatch(), because that's what its for."""
        if not self.allowed:
            if not request.user.is_authenticated:
                return http.HttpResponseRedirect(
                    self.login_url + '?next=' + request.path_info
                )
            else:
                return http.HttpResponseForbidden()
        return super().dispatch(request, *args, **kwargs)

    @classmethod
    def factory(cls, view, **attributes):
        if isinstance(view, str):
            view = import_string(view)
        return type(view.__name__, (view, cls), attributes)



from crudlfap import crudlfap

from .models import Post


class PostRouter(crudlfap.Router):
    fields = '__all__'
    icon = 'music'
    model = Post

    views = [
        crudlfap.DeleteView,
        crudlfap.UpdateView,
        crudlfap.CreateView,
        crudlfap.DetailView,
        crudlfap.FilterTables2ListView.clone(
            filter_fields=['owner'],
            search_fields=['name'],
        ),
    ]

    def allowed(self, view):
        user = view.request.user
        perms = view.required_permissions

        if perms == ['blog.add_post']:
            return user.is_authenticated

        if perms in [['blog.change_post'], ['blog.delete_post']]:
            return view.object.editable(user)

        return True

    def get_objects_for_user(self, user, perms):
        if perms in [['blog.change_post'], ['blog.delete_post']]:
            return self.model.objects.get_queryset().editable(user)
        return self.model.objects.get_queryset().readable(user)

PostRouter().register()
